@@ -5,9 +5,14 @@
   ___a__ | ( ___b__ << 8 ) | ( ___c__ << 16 ) | ( ___d__ << 24 ) | ( ___e__ << 32 ) | ( ___f__ << 40 ) | ( ___g__ << 48 ) | ( ___h__ << 56 )
  
  
+#define ___MAKE_INDEX_TBL128(___a__, ___b__, ___c__, ___d__, ___e__, ___f__, ___g__, ___h__,  ___i__, ___j__, ___k__, ___l__, ___m__, ___n__, ___o__, ___p__) \
+     ___a__ | ( ___b__ << 8L ) | ( ___c__ << 16L ) | ( ___d__ << 24L ) | ( ___e__ << 32L ) | ( ___f__ << 40L ) | ( ___g__ << 48L ) | ( ___h__ << 56L )  \
+  ,  ___i__ | ( ___j__ << 8L ) | ( ___k__ << 16L ) | ( ___l__ << 24L ) | ( ___m__ << 32L ) | ( ___n__ << 40L ) | ( ___o__ << 48L ) | ( ___p__ << 56L ) 
+ 
+ 
 #if  defined(___PERM4_Hi_inv_Table__)
 
-#if 1  
+#  if 1  
     ___MAKE_INDEX_TBL32(0, 0, 0, 0)
   , ___MAKE_INDEX_TBL32(0, 0, 0, 0)
   , ___MAKE_INDEX_TBL32(0, 0, 0, 0)
@@ -24,7 +29,7 @@
   , ___MAKE_INDEX_TBL32(0, 0, 1, 2)
   , ___MAKE_INDEX_TBL32(0, 0, 1, 2)
   , ___MAKE_INDEX_TBL32(0, 1, 2, 3)
-#else
+#  else
       ___MAKE_INDEX_TBL32(0, 0, 0, 0)
     , ___MAKE_INDEX_TBL32(0, 0, 0, 0)
     , ___MAKE_INDEX_TBL32(0, 0, 0, 0)
@@ -303,7 +308,7 @@
   , ___MAKE_INDEX_TBL64(0L, 0L, 1L, 2L, 3L, 4L, 5L, 6L)
   , ___MAKE_INDEX_TBL64(0L, 0L, 1L, 2L, 3L, 4L, 5L, 6L)
   , ___MAKE_INDEX_TBL64(0L, 1L, 2L, 3L, 4L, 5L, 6L, 7L)
-#else
+#  else
     ___MAKE_INDEX_TBL64(0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L)
   , ___MAKE_INDEX_TBL64(0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L)
   , ___MAKE_INDEX_TBL64(0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L)
@@ -560,10 +565,97 @@
   , ___MAKE_INDEX_TBL64(0L, 0L, 1L, 2L, 3L, 4L, 5L, 6L)
   , ___MAKE_INDEX_TBL64(0L, 0L, 1L, 2L, 3L, 4L, 5L, 6L)
   , ___MAKE_INDEX_TBL64(0L, 1L, 2L, 3L, 4L, 5L, 6L, 7L)
-#endif
+#  endif
+ 
+#elif defined( ___ipv4GatherHiTbl__ )
+ 
+const uint64_t ipv4GatherHiTbl[]  __attribute__ ((aligned(8))) = {
+  ___MAKE_INDEX_TBL128(255L, 255L, 255L, 0L, 255L, 255L, 255L, 2L, 255L, 255L, 255L, 4L, 255L, 255L, 255L, 6L)
+, ___MAKE_INDEX_TBL128(255L, 255L, 0L, 1L, 255L, 255L, 255L, 3L, 255L, 255L, 255L, 5L, 255L, 255L, 255L, 7L)
+, ___MAKE_INDEX_TBL128(255L, 0L, 1L, 2L, 255L, 255L, 255L, 4L, 255L, 255L, 255L, 6L, 255L, 255L, 255L, 8L)
+, ___MAKE_INDEX_TBL128(255L, 255L, 255L, 0L, 255L, 255L, 2L, 3L, 255L, 255L, 255L, 5L, 255L, 255L, 255L, 7L)
+, ___MAKE_INDEX_TBL128(255L, 255L, 0L, 1L, 255L, 255L, 3L, 4L, 255L, 255L, 255L, 6L, 255L, 255L, 255L, 8L)
+, ___MAKE_INDEX_TBL128(255L, 0L, 1L, 2L, 255L, 255L, 4L, 5L, 255L, 255L, 255L, 7L, 255L, 255L, 255L, 9L)
+, ___MAKE_INDEX_TBL128(255L, 255L, 255L, 0L, 255L, 2L, 3L, 4L, 255L, 255L, 255L, 6L, 255L, 255L, 255L, 8L)
+, ___MAKE_INDEX_TBL128(255L, 255L, 0L, 1L, 255L, 3L, 4L, 5L, 255L, 255L, 255L, 7L, 255L, 255L, 255L, 9L)
+, ___MAKE_INDEX_TBL128(255L, 0L, 1L, 2L, 255L, 4L, 5L, 6L, 255L, 255L, 255L, 8L, 255L, 255L, 255L, 10L)
+, ___MAKE_INDEX_TBL128(255L, 255L, 255L, 0L, 255L, 255L, 255L, 2L, 255L, 255L, 4L, 5L, 255L, 255L, 255L, 7L)
+, ___MAKE_INDEX_TBL128(255L, 255L, 0L, 1L, 255L, 255L, 255L, 3L, 255L, 255L, 5L, 6L, 255L, 255L, 255L, 8L)
+, ___MAKE_INDEX_TBL128(255L, 0L, 1L, 2L, 255L, 255L, 255L, 4L, 255L, 255L, 6L, 7L, 255L, 255L, 255L, 9L)
+, ___MAKE_INDEX_TBL128(255L, 255L, 255L, 0L, 255L, 255L, 2L, 3L, 255L, 255L, 5L, 6L, 255L, 255L, 255L, 8L)
+, ___MAKE_INDEX_TBL128(255L, 255L, 0L, 1L, 255L, 255L, 3L, 4L, 255L, 255L, 6L, 7L, 255L, 255L, 255L, 9L)
+, ___MAKE_INDEX_TBL128(255L, 0L, 1L, 2L, 255L, 255L, 4L, 5L, 255L, 255L, 7L, 8L, 255L, 255L, 255L, 10L)
+, ___MAKE_INDEX_TBL128(255L, 255L, 255L, 0L, 255L, 2L, 3L, 4L, 255L, 255L, 6L, 7L, 255L, 255L, 255L, 9L)
+, ___MAKE_INDEX_TBL128(255L, 255L, 0L, 1L, 255L, 3L, 4L, 5L, 255L, 255L, 7L, 8L, 255L, 255L, 255L, 10L)
+, ___MAKE_INDEX_TBL128(255L, 0L, 1L, 2L, 255L, 4L, 5L, 6L, 255L, 255L, 8L, 9L, 255L, 255L, 255L, 11L)
+, ___MAKE_INDEX_TBL128(255L, 255L, 255L, 0L, 255L, 255L, 255L, 2L, 255L, 4L, 5L, 6L, 255L, 255L, 255L, 8L)
+, ___MAKE_INDEX_TBL128(255L, 255L, 0L, 1L, 255L, 255L, 255L, 3L, 255L, 5L, 6L, 7L, 255L, 255L, 255L, 9L)
+, ___MAKE_INDEX_TBL128(255L, 0L, 1L, 2L, 255L, 255L, 255L, 4L, 255L, 6L, 7L, 8L, 255L, 255L, 255L, 10L)
+, ___MAKE_INDEX_TBL128(255L, 255L, 255L, 0L, 255L, 255L, 2L, 3L, 255L, 5L, 6L, 7L, 255L, 255L, 255L, 9L)
+, ___MAKE_INDEX_TBL128(255L, 255L, 0L, 1L, 255L, 255L, 3L, 4L, 255L, 6L, 7L, 8L, 255L, 255L, 255L, 10L)
+, ___MAKE_INDEX_TBL128(255L, 0L, 1L, 2L, 255L, 255L, 4L, 5L, 255L, 7L, 8L, 9L, 255L, 255L, 255L, 11L)
+, ___MAKE_INDEX_TBL128(255L, 255L, 255L, 0L, 255L, 2L, 3L, 4L, 255L, 6L, 7L, 8L, 255L, 255L, 255L, 10L)
+, ___MAKE_INDEX_TBL128(255L, 255L, 0L, 1L, 255L, 3L, 4L, 5L, 255L, 7L, 8L, 9L, 255L, 255L, 255L, 11L)
+, ___MAKE_INDEX_TBL128(255L, 0L, 1L, 2L, 255L, 4L, 5L, 6L, 255L, 8L, 9L, 10L, 255L, 255L, 255L, 12L)
+, ___MAKE_INDEX_TBL128(255L, 255L, 255L, 0L, 255L, 255L, 255L, 2L, 255L, 255L, 255L, 4L, 255L, 255L, 6L, 7L)
+, ___MAKE_INDEX_TBL128(255L, 255L, 0L, 1L, 255L, 255L, 255L, 3L, 255L, 255L, 255L, 5L, 255L, 255L, 7L, 8L)
+, ___MAKE_INDEX_TBL128(255L, 0L, 1L, 2L, 255L, 255L, 255L, 4L, 255L, 255L, 255L, 6L, 255L, 255L, 8L, 9L)
+, ___MAKE_INDEX_TBL128(255L, 255L, 255L, 0L, 255L, 255L, 2L, 3L, 255L, 255L, 255L, 5L, 255L, 255L, 7L, 8L)
+, ___MAKE_INDEX_TBL128(255L, 255L, 0L, 1L, 255L, 255L, 3L, 4L, 255L, 255L, 255L, 6L, 255L, 255L, 8L, 9L)
+, ___MAKE_INDEX_TBL128(255L, 0L, 1L, 2L, 255L, 255L, 4L, 5L, 255L, 255L, 255L, 7L, 255L, 255L, 9L, 10L)
+, ___MAKE_INDEX_TBL128(255L, 255L, 255L, 0L, 255L, 2L, 3L, 4L, 255L, 255L, 255L, 6L, 255L, 255L, 8L, 9L)
+, ___MAKE_INDEX_TBL128(255L, 255L, 0L, 1L, 255L, 3L, 4L, 5L, 255L, 255L, 255L, 7L, 255L, 255L, 9L, 10L)
+, ___MAKE_INDEX_TBL128(255L, 0L, 1L, 2L, 255L, 4L, 5L, 6L, 255L, 255L, 255L, 8L, 255L, 255L, 10L, 11L)
+, ___MAKE_INDEX_TBL128(255L, 255L, 255L, 0L, 255L, 255L, 255L, 2L, 255L, 255L, 4L, 5L, 255L, 255L, 7L, 8L)
+, ___MAKE_INDEX_TBL128(255L, 255L, 0L, 1L, 255L, 255L, 255L, 3L, 255L, 255L, 5L, 6L, 255L, 255L, 8L, 9L)
+, ___MAKE_INDEX_TBL128(255L, 0L, 1L, 2L, 255L, 255L, 255L, 4L, 255L, 255L, 6L, 7L, 255L, 255L, 9L, 10L)
+, ___MAKE_INDEX_TBL128(255L, 255L, 255L, 0L, 255L, 255L, 2L, 3L, 255L, 255L, 5L, 6L, 255L, 255L, 8L, 9L)
+, ___MAKE_INDEX_TBL128(255L, 255L, 0L, 1L, 255L, 255L, 3L, 4L, 255L, 255L, 6L, 7L, 255L, 255L, 9L, 10L)
+, ___MAKE_INDEX_TBL128(255L, 0L, 1L, 2L, 255L, 255L, 4L, 5L, 255L, 255L, 7L, 8L, 255L, 255L, 10L, 11L)
+, ___MAKE_INDEX_TBL128(255L, 255L, 255L, 0L, 255L, 2L, 3L, 4L, 255L, 255L, 6L, 7L, 255L, 255L, 9L, 10L)
+, ___MAKE_INDEX_TBL128(255L, 255L, 0L, 1L, 255L, 3L, 4L, 5L, 255L, 255L, 7L, 8L, 255L, 255L, 10L, 11L)
+, ___MAKE_INDEX_TBL128(255L, 0L, 1L, 2L, 255L, 4L, 5L, 6L, 255L, 255L, 8L, 9L, 255L, 255L, 11L, 12L)
+, ___MAKE_INDEX_TBL128(255L, 255L, 255L, 0L, 255L, 255L, 255L, 2L, 255L, 4L, 5L, 6L, 255L, 255L, 8L, 9L)
+, ___MAKE_INDEX_TBL128(255L, 255L, 0L, 1L, 255L, 255L, 255L, 3L, 255L, 5L, 6L, 7L, 255L, 255L, 9L, 10L)
+, ___MAKE_INDEX_TBL128(255L, 0L, 1L, 2L, 255L, 255L, 255L, 4L, 255L, 6L, 7L, 8L, 255L, 255L, 10L, 11L)
+, ___MAKE_INDEX_TBL128(255L, 255L, 255L, 0L, 255L, 255L, 2L, 3L, 255L, 5L, 6L, 7L, 255L, 255L, 9L, 10L)
+, ___MAKE_INDEX_TBL128(255L, 255L, 0L, 1L, 255L, 255L, 3L, 4L, 255L, 6L, 7L, 8L, 255L, 255L, 10L, 11L)
+, ___MAKE_INDEX_TBL128(255L, 0L, 1L, 2L, 255L, 255L, 4L, 5L, 255L, 7L, 8L, 9L, 255L, 255L, 11L, 12L)
+, ___MAKE_INDEX_TBL128(255L, 255L, 255L, 0L, 255L, 2L, 3L, 4L, 255L, 6L, 7L, 8L, 255L, 255L, 10L, 11L)
+, ___MAKE_INDEX_TBL128(255L, 255L, 0L, 1L, 255L, 3L, 4L, 5L, 255L, 7L, 8L, 9L, 255L, 255L, 11L, 12L)
+, ___MAKE_INDEX_TBL128(255L, 0L, 1L, 2L, 255L, 4L, 5L, 6L, 255L, 8L, 9L, 10L, 255L, 255L, 12L, 13L)
+, ___MAKE_INDEX_TBL128(255L, 255L, 255L, 0L, 255L, 255L, 255L, 2L, 255L, 255L, 255L, 4L, 255L, 6L, 7L, 8L)
+, ___MAKE_INDEX_TBL128(255L, 255L, 0L, 1L, 255L, 255L, 255L, 3L, 255L, 255L, 255L, 5L, 255L, 7L, 8L, 9L)
+, ___MAKE_INDEX_TBL128(255L, 0L, 1L, 2L, 255L, 255L, 255L, 4L, 255L, 255L, 255L, 6L, 255L, 8L, 9L, 10L)
+, ___MAKE_INDEX_TBL128(255L, 255L, 255L, 0L, 255L, 255L, 2L, 3L, 255L, 255L, 255L, 5L, 255L, 7L, 8L, 9L)
+, ___MAKE_INDEX_TBL128(255L, 255L, 0L, 1L, 255L, 255L, 3L, 4L, 255L, 255L, 255L, 6L, 255L, 8L, 9L, 10L)
+, ___MAKE_INDEX_TBL128(255L, 0L, 1L, 2L, 255L, 255L, 4L, 5L, 255L, 255L, 255L, 7L, 255L, 9L, 10L, 11L)
+, ___MAKE_INDEX_TBL128(255L, 255L, 255L, 0L, 255L, 2L, 3L, 4L, 255L, 255L, 255L, 6L, 255L, 8L, 9L, 10L)
+, ___MAKE_INDEX_TBL128(255L, 255L, 0L, 1L, 255L, 3L, 4L, 5L, 255L, 255L, 255L, 7L, 255L, 9L, 10L, 11L)
+, ___MAKE_INDEX_TBL128(255L, 0L, 1L, 2L, 255L, 4L, 5L, 6L, 255L, 255L, 255L, 8L, 255L, 10L, 11L, 12L)
+, ___MAKE_INDEX_TBL128(255L, 255L, 255L, 0L, 255L, 255L, 255L, 2L, 255L, 255L, 4L, 5L, 255L, 7L, 8L, 9L)
+, ___MAKE_INDEX_TBL128(255L, 255L, 0L, 1L, 255L, 255L, 255L, 3L, 255L, 255L, 5L, 6L, 255L, 8L, 9L, 10L)
+, ___MAKE_INDEX_TBL128(255L, 0L, 1L, 2L, 255L, 255L, 255L, 4L, 255L, 255L, 6L, 7L, 255L, 9L, 10L, 11L)
+, ___MAKE_INDEX_TBL128(255L, 255L, 255L, 0L, 255L, 255L, 2L, 3L, 255L, 255L, 5L, 6L, 255L, 8L, 9L, 10L)
+, ___MAKE_INDEX_TBL128(255L, 255L, 0L, 1L, 255L, 255L, 3L, 4L, 255L, 255L, 6L, 7L, 255L, 9L, 10L, 11L)
+, ___MAKE_INDEX_TBL128(255L, 0L, 1L, 2L, 255L, 255L, 4L, 5L, 255L, 255L, 7L, 8L, 255L, 10L, 11L, 12L)
+, ___MAKE_INDEX_TBL128(255L, 255L, 255L, 0L, 255L, 2L, 3L, 4L, 255L, 255L, 6L, 7L, 255L, 9L, 10L, 11L)
+, ___MAKE_INDEX_TBL128(255L, 255L, 0L, 1L, 255L, 3L, 4L, 5L, 255L, 255L, 7L, 8L, 255L, 10L, 11L, 12L)
+, ___MAKE_INDEX_TBL128(255L, 0L, 1L, 2L, 255L, 4L, 5L, 6L, 255L, 255L, 8L, 9L, 255L, 11L, 12L, 13L)
+, ___MAKE_INDEX_TBL128(255L, 255L, 255L, 0L, 255L, 255L, 255L, 2L, 255L, 4L, 5L, 6L, 255L, 8L, 9L, 10L)
+, ___MAKE_INDEX_TBL128(255L, 255L, 0L, 1L, 255L, 255L, 255L, 3L, 255L, 5L, 6L, 7L, 255L, 9L, 10L, 11L)
+, ___MAKE_INDEX_TBL128(255L, 0L, 1L, 2L, 255L, 255L, 255L, 4L, 255L, 6L, 7L, 8L, 255L, 10L, 11L, 12L)
+, ___MAKE_INDEX_TBL128(255L, 255L, 255L, 0L, 255L, 255L, 2L, 3L, 255L, 5L, 6L, 7L, 255L, 9L, 10L, 11L)
+, ___MAKE_INDEX_TBL128(255L, 255L, 0L, 1L, 255L, 255L, 3L, 4L, 255L, 6L, 7L, 8L, 255L, 10L, 11L, 12L)
+, ___MAKE_INDEX_TBL128(255L, 0L, 1L, 2L, 255L, 255L, 4L, 5L, 255L, 7L, 8L, 9L, 255L, 11L, 12L, 13L)
+, ___MAKE_INDEX_TBL128(255L, 255L, 255L, 0L, 255L, 2L, 3L, 4L, 255L, 6L, 7L, 8L, 255L, 10L, 11L, 12L)
+, ___MAKE_INDEX_TBL128(255L, 255L, 0L, 1L, 255L, 3L, 4L, 5L, 255L, 7L, 8L, 9L, 255L, 11L, 12L, 13L)
+, ___MAKE_INDEX_TBL128(255L, 0L, 1L, 2L, 255L, 4L, 5L, 6L, 255L, 8L, 9L, 10L, 255L, 12L, 13L, 14L)
+};
  
 #endif
  
  
 #undef ___MAKE_INDEX_TBL64
+#undef ___MAKE_INDEX_TBL128
  
